@@ -119,33 +119,6 @@ extern int rand();
 typedef uint32_t uint32n;
 typedef int32_t int32n;
 
-#include <fcntl.h>
-#include <time.h>
-#include <stdlib.h>
-#include <unistd.h>
-/* 
-   Before Netpbm 9.0, atoi() and exit() were declared for everybody
-   except MSDOS and Amiga, and time() and write() were declared for
-   everybody except MSDOS, Amiga, and __osf__.  fcntl.h, time.h, and
-   stlib.h were included for MSDOS and Amiga, and unistd.h was included
-   for everyone except VMS, MSDOS, and Amiga.  With the netbsd patches,
-   atoi(), exit(), time(), and write() were not declared for __NetBSD__.
-
-   We're hoping that all current systems have the standard header
-   files, and will reinstate some of these explicit declarations if we
-   hear otherwise.  
-
-   If it turns out to be this easy, we should just move these inclusions
-   to the source files that actually need them.
-   
-   -Bryan 2000.04.13
-
-extern int atoi();
-extern void exit();
-extern long time();
-extern int write(); 
-*/
-
 /* CONFIGURE: On most BSD systems, malloc() gets declared in stdlib.h, on
 ** system V, it gets declared in malloc.h. On some systems, malloc.h
 ** doesn't declare these, so we have to do it here. On other systems,
@@ -222,6 +195,26 @@ extern int write();
   #endif
 #endif
 
+/* At least one compiler can't handle two declarations of the same function
+   that aren't literally identical.  E.g. "static foo_fn_t foo1;" conflicts
+   with "static void foo1(int);" even if type 'foo_fn_t' is defined as
+   void(int).  (The compiler we saw do this is SGI IDO cc (for IRIX 4.3)).
+
+   LITERAL_FN_DEF_MATCH says that the compiler might have this problem,
+   so one must be conservative in redeclaring functions.
+*/
+#if defined(__GNUC__)
+  #define LITERAL_FN_DEF_MATCH 0
+#else
+  #if (defined(__sgi))
+    #define LITERAL_FN_DEF_MATCH 1
+  #else   
+    #define LITERAL_FN_DEF_MATCH 0
+  #endif
+#endif
+
+
+
 /* CONFIGURE: Some systems seem to need more than standard program linkage
    to get a data (as opposed to function) item out of a library.
 
@@ -243,6 +236,8 @@ extern int write();
 # endif
 #endif
 
+#include <unistd.h>  /* Get _LFS_LARGEFILE defined */
+#include <sys/types.h>
 /* In GNU, _LFS_LARGEFILE means the "off_t" functions (ftello, etc.) are
    available.  In AIX, _AIXVERSION_430 means it's AIX Version 4.3.0 or
    better, which seems to mean the "off_t" functions are available.
@@ -276,3 +271,5 @@ typedef long int pm_filepos;
   #define HAVE_MKSTEMP 1
 #endif
 
+typedef int qsort_comparison_fn(const void *, const void *);
+    /* A compare function to pass to <stdlib.h>'s qsort() */
